@@ -26,45 +26,22 @@
 
 #include <libgit2-glib/ggit-types.h>
 #include <libgit2-glib/ggit-native.h>
+#include <libgit2-glib/ggit-remote-callbacks.h>
 
 G_BEGIN_DECLS
 
-#define GGIT_TYPE_REMOTE		(ggit_remote_get_type ())
-#define GGIT_REMOTE(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GGIT_TYPE_REMOTE, GgitRemote))
-#define GGIT_REMOTE_CONST(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GGIT_TYPE_REMOTE, GgitRemote const))
-#define GGIT_REMOTE_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GGIT_TYPE_REMOTE, GgitRemoteClass))
-#define GGIT_IS_REMOTE(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GGIT_TYPE_REMOTE))
-#define GGIT_IS_REMOTE_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GGIT_TYPE_REMOTE))
-#define GGIT_REMOTE_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GGIT_TYPE_REMOTE, GgitRemoteClass))
-
-typedef struct _GgitRemoteClass		GgitRemoteClass;
-typedef struct _GgitRemotePrivate	GgitRemotePrivate;
-
-struct _GgitRemote
-{
-	/*< private >*/
-	GgitNative parent;
-
-	/*< priv padding >*/
-	GgitRemotePrivate *priv;
-};
+#define GGIT_TYPE_REMOTE (ggit_remote_get_type ())
+G_DECLARE_DERIVABLE_TYPE (GgitRemote, ggit_remote, GGIT, REMOTE, GgitNative)
 
 struct _GgitRemoteClass
 {
 	/*< private >*/
 	GgitNativeClass parent_class;
-
-	/* < signals > */
-	void (*tip_updated) (GgitRemote    *remote,
-	                     const gchar   *refname,
-	                     const GgitOId *a,
-	                     const GgitOId *b);
 };
 
 #define GGIT_TYPE_REMOTE_HEAD   (ggit_remote_head_get_type ())
 #define GGIT_REMOTE_HEAD(obj)        ((GgitRemoteHead *)obj)
 
-GType              ggit_remote_get_type                 (void) G_GNUC_CONST;
 GType              ggit_remote_head_get_type            (void) G_GNUC_CONST;
 
 GgitRemoteHead    *ggit_remote_head_ref                 (GgitRemoteHead *remote_head);
@@ -75,6 +52,8 @@ GgitOId           *ggit_remote_head_get_local_oid       (GgitRemoteHead *remote_
 gboolean           ggit_remote_head_is_local            (GgitRemoteHead *remote_head);
 const gchar       *ggit_remote_head_get_name            (GgitRemoteHead *remote_head);
 
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (GgitRemoteHead, ggit_remote_head_unref)
+
 GgitRemote       *_ggit_remote_wrap                     (git_remote     *remote);
 
 GgitRemote        *ggit_remote_new                      (GgitRepository   *repository,
@@ -84,21 +63,18 @@ GgitRemote        *ggit_remote_new                      (GgitRepository   *repos
 
 GgitRemote        *ggit_remote_new_anonymous            (GgitRepository   *repository,
                                                          const gchar      *url,
-                                                         const gchar      *fetch,
                                                          GError          **error);
 
 GgitRepository    *ggit_remote_get_owner                (GgitRemote       *remote);
 
-void               ggit_remote_save                     (GgitRemote       *remote,
-                                                         GError          **error);
-
 const gchar       *ggit_remote_get_name                 (GgitRemote       *remote);
 
 const gchar       *ggit_remote_get_url                  (GgitRemote       *remote);
 
-void               ggit_remote_connect                  (GgitRemote       *remote,
-                                                         GgitDirection     direction,
-                                                         GError          **error);
+void               ggit_remote_connect                  (GgitRemote           *remote,
+                                                         GgitDirection         direction,
+                                                         GgitRemoteCallbacks  *callbacks,
+                                                         GError              **error);
 
 gboolean           ggit_remote_get_connected            (GgitRemote       *remote);
 
@@ -106,44 +82,25 @@ void               ggit_remote_disconnect               (GgitRemote       *remot
 
 gboolean           ggit_remote_download                 (GgitRemote           *remote,
                                                          const gchar * const  *specs,
+                                                         GgitFetchOptions     *fetch_options,
                                                          GError              **error);
 
-gboolean           ggit_remote_update_tips              (GgitRemote       *remote,
-                                                         GgitSignature    *signature,
-                                                         const gchar      *message,
-                                                         GError          **error);
-
-void               ggit_remote_add_fetch_spec           (GgitRemote       *remote,
-                                                         const gchar      *fetch_spec,
-                                                         GError          **error);
+gboolean           ggit_remote_update_tips              (GgitRemote                  *remote,
+                                                         GgitRemoteCallbacks         *callbacks,
+                                                         gboolean                     update_fetch_head,
+                                                         GgitRemoteDownloadTagsType   tags_type,
+                                                         const gchar                 *message,
+                                                         GError                     **error);
 
 gchar            **ggit_remote_get_fetch_specs          (GgitRemote       *remote,
                                                          GError          **error);
 
-gboolean           ggit_remote_set_fetch_specs          (GgitRemote           *remote,
-                                                         const gchar * const  *specs,
-                                                         GError              **error);
-
-void               ggit_remote_add_push_spec            (GgitRemote       *remote,
-                                                         const gchar      *push_spec,
-                                                         GError          **error);
-
 gchar            **ggit_remote_get_push_specs           (GgitRemote       *remote,
                                                          GError          **error);
 
-gboolean           ggit_remote_set_push_specs           (GgitRemote           *remote,
-                                                         const gchar * const  *specs,
-                                                         GError              **error);
-
 GgitRemoteHead   **ggit_remote_list                     (GgitRemote       *remote,
                                                          GError          **error);
 
-GgitRemoteCallbacks *
-                   ggit_remote_get_callbacks            (GgitRemote       *remote);
-
-void               ggit_remote_set_callbacks            (GgitRemote       *remote,
-                                                         GgitRemoteCallbacks *callbacks);
-
 G_END_DECLS
 
 #endif /* __GGIT_REMOTE_H__ */
